/*
Author: Ampersand
Create a keyframe animation.

Example:
[_timelineData, _curvesData] call compile preprocessFileLineNumbers "keyframe.sqf";

Arguments:
0: Timeline Data <ARRAY> List of pairs holding variable names and values representing the attributes on the Timeline module
	
	Example:
	private _timelineData = [
		["Length", 10],
		["Loop", true]
	];

	Timeline Attributes:
	["Pos", 0.0] <NUMBER> The initial position
	["Length", 10.0] <NUMBER> The length in seconds
	["PlayRate", 1.0] <NUMBER> The play rate
	["Loop", false] <BOOLEAN> Whether loop is enabled
	["Reverse", false] <BOOLEAN> Whether we should play backwards
	["PlayFromStart", false] <BOOLEAN> Whether to start playing at scenario start
	["DestroyWhenFinished", false] <BOOLEAN> Whether to be destroyed when finishing playing
	["InterpMode", 0] <NUMBER> Interpolation mode
		* 0  - Linear
		* 1  - Cubic
		* 2  - EaseIn
		* 3  - EaseOut
		* 4  - EaseInOut
		* 5  - Hermite
		* 6  - Berp
		* 7  - BounceIn
		* 8  - BounceOut
		* 9  - BounceInOut
		* 10 - QuinticIn
		* 11 - QuinticOut
		* 12 - QuinticInOut
		* 13 - ElasticIn
		* 14 - ElasticOut
		* 15 - ElasticInOut

1: Curves Data <ARRAY> Array of 1 or more curves
	
	Example:
	private _curvesData = [
		_curve1,
		_curve2
	];

	Arguments for a single Curve:
	0: Simulated Objects <ARRAY> List of objects simulated by given curve
	1: Curve Attributes <ARRAY> List of pairs holding variable names and values representing the attributes on the Curve module
	2: Keys <ARRAY> List of keys associated with the given curve

	Curve Attributes:
	["OrientationMode", 0] <NUMBER> The orientation mode (0 = None / 1 = Animation / 2 = Look At / 3 = Movement Direction)
	["ConstantSpeed", false] <BOOLEAN> True if speed is constant when moving along this curve, false if not
	["TimeInSeconds", true] <BOOLEAN> True if in seconds, false if in alpha
	
	Example:
	private _curve1 = [
		[_vehicle1, _vehicle2], // Objects
		[ // Curve variables
			["OrientationMode", 3],
			["TimeInSeconds", false]
		],
		_keys
	];

Keys

	Example:
	private _keys = [_key1, _key2];

	Arguments for a single Key:
	0: Time <NUMBER>
	1: Position ASL <ARRAY>
	2: Orientation <ARRAY> [_vectorDir, _vectorUp]
	3: Key Attributes <ARRAY> List of pairs holding variable names and values representing the attributes on the Key module
	
	Example:
	private _key1 = [
		0,
		getPosASL _vehicle1,
		[vectorDir _vehicle1, vectorUp _vehicle1],
		[
			["LeaveTangent", vectorDir _vehicle1],
			["LeaveTangentWeight", 2.0]
		]
	];

	Key Attributes:
		["KeyTime", 0] <NUMBER>
		["InterpMode", 0] <NUMBER> Interpolation mode
		["ArriveTangent", [0.0, 0.0, 0.0]] <ARRAY>
		["ArriveTangentWeight", 1.0] <NUMBER>
		["LeaveTangent", [0.0, 0.0, 0.0]] <ARRAY>
		["LeaveTangentWeight", 1.0] <NUMBER>
		["Value", 0.0] <NUMBER> ?

Control point ControlPoint_F
"TangentType", true Bool - Whether this control point is arrive or leave

Testing Snippets:

{
	// Current result is saved in variable _x
	deleteVehicle _x;
} forEach (units a_group);
deleteGroup a_group;
deleteVehicle a_vehicle;
a_vehicle = createVehicle ["O_T_UAV_04_CAS_F", [0,0,0], [], 0, "CAN_COLLIDE"];
a_vehicle allowDamage false;

private _curvesData = [n
	[ // curve1
		[a_vehicle], // vehicles
		[ // Curve variables
			["OrientationMode", 1],
			["TimeInSeconds", false]
		],
		[ // Curve keys
			[0, [0,0,10], [[0,1,0], [0,0,1]], []],
			[5, [0,10,15], [[1,0,0], [0,1,0]], []],
			[10, [10,10,20], [[0,-1,0], [1,0,0]], []]
		]
	]
];


_v = call amp_fnc_gv; allVariables _v select {!("cba" in _x)} apply {[_x, _v getVariable [_x, "e"]]}

private _timelineData = [ // Timeline attributes/variables
	["Length", 10],
	["Loop", true]
];
[_timelineData, _curvesData] call compile preprocessFileLineNumbers "keyframe.sqf";

---------------------------------------------------------------------------- */

params ["_timelineData", "_curvesData"];

private _group = createGroup [sideLogic, true];
a_group = _group;
private _timeline = _group createUnit ["Timeline_F", [0,0,0], [], 0, "CAN_COLLIDE"];
{
	_timeline setVariable _x;
} forEach _timelineData;

{
	_x params ["_objects", "_variables", "_keysData"];
	private _firstPos = _keysData param [0, [0, [0,0,0]]] param [1, [0,0,0]];
	_modulePos = +_firstPos;
	_modulePos set [2, 0];
	if (_forEachIndex == 1) then {_timeline setPos (_modulePos vectorAdd [-20,0,0])};

	private _curve = _group createUnit ["Curve_F", _modulePos vectorAdd [-10,10*_forEachIndex,0], [], 0, "CAN_COLLIDE"];
	_timeline synchronizeObjectsAdd [_curve];
	_curve synchronizeObjectsAdd _objects;
	{
		_curve setVariable _x;
	} forEach _variables;
	
	{
		_x params ["_time", "_pos", "_vDirUp", ["_variables", []]];
		private _key = _group createUnit ["Key_F", [0,0,0], [], 0, "CAN_COLLIDE"];
		_key setPosASL _pos;
		_key setVectorDirAndUp _vDirUp;
		
		{
			_key setVariable _x;
		} forEach _variables;
		_key setVariable ["KeyTime", _time];
		_curve synchronizeObjectsAdd [_key];
	} forEach _keysData;
	
} forEach _curvesData;

//_timeline setVariable ["loop", true];

{
	_x addCuratorEditableObjects [units _group];
} forEach allCurators;

// Iterate simulated curves
{
	// Compute curve
	[_x, true] call BIS_fnc_richCurve_compute;
} forEach ([_timeline] call BIS_fnc_timeline_getSimulatedCurves);

[_timeline] call BIS_fnc_timeline_play;
_timeline;